# coding=utf-8
import yaml


from typing import Any
TyAny = Any
TyArr = list[Any]
TyDic = dict[Any, Any]
TyPath = str
TyYaml = int | str | float | TyDic | TyArr
TnYaml = None | TyYaml

TnAny = None | Any


class Yaml_:
    """ Manage Object to Yaml file affilitation
    """
    @staticmethod
    def read_with_safeloader(path: TyPath) -> TnAny:
        try:
            with open(path) as fd:
                # The Loader parameter handles the conversion from YAML
                # scalar values to Python object format
                obj = yaml.load(fd, Loader=yaml.SafeLoader)
                return obj
        except FileNotFoundError:
            msg = f"No such file or directory: path='{path}'"
            raise Exception(msg)
        except IOError:
            raise
        return None

    @staticmethod
    def write(path: TyPath, obj: Any) -> None:
        with open(path, 'w') as fd:
            yaml.dump(
                obj, fd,
                Dumper=yaml.SafeDumper,
                sort_keys=False,
                indent=4,
                default_flow_style=False
            )



# coding=utf-8
from typing import Any

import inspect

TyClsName = Any
TyModName = Any
TyPacName = Any
TyPacModName = Any
TyFncName = Any


class Fnc:

    @staticmethod
    def sh_pacmod_name(fnc) -> TyPacModName:
        """
        show module name of function
        """
        return fnc.__module__

    @staticmethod
    def sh_mod_name(fnc) -> TyModName:
        """
        show module name of function
        """
        _mod_name = fnc.__module__
        parts = _mod_name.split('.')
        if len(parts) > 1:
            _mod_name = parts[1]
        return _mod_name

    @staticmethod
    def sh_pac_name(fnc) -> TyModName:
        """
        show module name of function
        """
        _mod_name = fnc.__module__
        parts = _mod_name.split('.')
        if len(parts) > 1:
            _pac_name = parts[0]
        else:
            _pac_name = ''
        return _pac_name

    @staticmethod
    def xsh_pac_name(fnc) -> TyPacName:
        """
        show package name of function
        """
        # Get the package name (if available)
        _mod = inspect.getmodule(fnc)
        _pac_name = _mod.__package__ if _mod else ''
        return _pac_name

    @staticmethod
    def sh_cls_name(fnc) -> TyClsName:
        """
        show class name of function
        """
        if hasattr(fnc, '__qualname__'):
            parts = fnc.__qualname__.split('.')
            if len(parts) > 1:
                _cls_name = parts[0]  # Class name
        else:
            _cls_name = ''
        return _cls_name

    @classmethod
    def sh_fnc_name(cls, fnc) -> TyFncName:
        """
        show class name of function
        """
        _pacmod_name = cls.sh_pacmod_name(fnc)
        _cls_name = cls.sh_cls_name(fnc)
        _fnc_name = '.'.join([_pacmod_name, _cls_name])
        return _fnc_name



# coding=utf-8
from typing import Any

import os

# from ka_uts_uts.utils.pac import Pac

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPath = str
TyStr = str
TyTup = tuple[Any, Any]


class Path:
    """ Package Module Management
    """
    # @staticmethod
    # def sh_t_pacmod(cls) -> TyTup:
    #     """ Show Pacmod Dictionary
    #     """
    #     a_pacmod: TyArr = cls.__module__.split(".")
    #     return tuple(a_pacmod)

    # @staticmethod
    # def sh_d_pacmod(cls) -> TyDic:
    #     """ Show Pacmod Dictionary
    #     """
    #     a_pacmod: TyArr = cls.__module__.split(".")
    #     return {'package': a_pacmod[0], 'module': a_pacmod[1:]}

    # @staticmethod
    # def sh_path_module_yaml(d_pacmod: TyDic) -> Any:
    #     """ show directory
    #     """
    #     _package = d_pacmod['package']
    #     _package = cls_com.__module__.split(".")[0]
    #     _module = d_pacmod['module']
    #     _path = f"{_module}/data/{_module}.yml"
    #     return Pac.sh_path_by_path(_package, _path)

    # @classmethod
    # def sh_path_by_data_keys_yml(cls, cls_com) -> Any:
    #     """
    #     show path to configuration yaml-file in data directory of package
    #     """
    #     _package = cls_com.__module__.split(".")[0]
    #     _path = os.path.join('data', 'keys.yml')
    #     return Pac.sh_path_by_path(_package, _path)

    # @classmethod
    # def sh_path_cfg_yml(cls, cls_app) -> Any:
    #     """
    #     show path to configuration yaml-file in data directory of package
    #     """
    #     _d_pacmod = cls.sh_d_pacmod(cls_app)
    #     _package = _d_pacmod['package']
    #     _path = os.path.join('data', 'cfg.yml')
    #     return Pac.sh_path_by_path(_package, _path)

    # @classmethod
    # def sh_dir_type(cls, kwargs: TyDic, type_: TyStr) -> TyPath:
    #     """
    #     Show run_dir
    #     """
    #     app_com = kwargs.get('app_com')
    #     app_data = kwargs.get('app_data', '')
    #     tenant = kwargs.get('tenant', '')
    #     a_pacmod: TyArr = app_com.__module__.split(".")
    #     package = a_pacmod[0]
    #     module = a_pacmod[1]
    #     return os.path.join(app_data, tenant, package, module, type_)

    @staticmethod
    def sh_app_data_path_by_type_and_file_pattern(
            kwargs: TyDic, filename, type_: str, suffix: str) -> TyPath:
        """
        show type specific path
        """
        _app_com = kwargs.get('app_com')
        _app_data = kwargs.get('app_data', '')
        _tenant = kwargs.get('tenant', '')
        _a_pacmod: TyArr = _app_com.__module__.split(".")
        _package = _a_pacmod[0]
        _module = _a_pacmod[1]
        _path = os.path.join(_app_data, _tenant, _package, _module, type_)
        return os.path.join(_path, f"{filename}*.{suffix}")



__title__ = 'ka_uts_uts'
__description__ = 'IO Control & Package Management Utilities.'
__url__ = 'https://ka-uts-uts.readthedocs.io/en/latest'
__version__ = '4.0.3.250519'
__build__ = 0x022200
__author__ = 'Bernd Stroehle'
__author_email__ = 'bernd.stroehle@gmail.com'
__license__ = 'GPL-3.0-only WITH Classpath-Exception-2.0 OR BSD-3-Clause'
__copyright__ = 'Copyright 2025 Bernd Stroehle'
__cake__ = u'\u2728 \U0001f370 \u2728'



# coding=utf-8
from typing import Any

import os
from ka_uts_uts.utils.pac import Pac

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPackage = str
TyPath = str
TnPath = None | TyPath


class Cls:

    @staticmethod
    def sh_path_of_pac_by_path(cls_app, log, path: TyPath) -> Any:
        # def sh_path_in_cls(cls, path: TyPath) -> Any:
        """
        show directory
        """
        _package = cls_app.__module__.split(".")[0]
        return Pac.sh_path_by_path(_package, path, log)

    @staticmethod
    def sh_path_of_pac_by_paths(cls_app, log, *paths: TyPath) -> Any:
        # def sh_path_in_cls(cls, path: TyPath) -> Any:
        """
        show directory
        """
        _package = cls_app.__module__.split(".")[0]
        _path = os.sep.join(paths)
        return Pac.sh_path_by_path(_package, log, _path)



# coding=utf-8
from typing import Any

import os

from ka_uts_uts.utils.pac import Pac

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPackage = str
TyPackages = list[str]
TyPath = str


class Pacs:

    @staticmethod
    def sh_path_by_path(
            packages: TyPackages, path: TyPath, log) -> Any:
        """ show directory
        """
        if not isinstance(packages, list):
            packages = [packages]
        for _package in packages:
            _path = Pac.sh_path_by_path(_package, path, log)
            if _path:
                return _path
        return ''

    @classmethod
    def sh_path_by_path_and_prefix(
            cls, packages: TyPackages, path: TyPath, log, prefix: TyPath = '') -> Any:
        # def sh_path_by_packs(
        """ show directory
        """
        if prefix:
            _path = os.path.join(prefix, path)
            # _dirname = os.path.dirname(_path)
            if os.path.exists(_path):
                return _path
        return cls.sh_path_by_path(packages, path, log)



# coding=utf-8
from typing import Any

import os
import importlib.resources

# from ka_uts_log.log import LogEq
# from ka_uts_log.log import Log

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPackage = str
TyPackages = list[str]
TyPath = str
TnPath = None | TyPath


class Pac:

    @staticmethod
    def sh_path(package: TyPackage) -> Any:
        return str(importlib.resources.files(package))

    @staticmethod
    def sh_path_by_path(
            package: TyPackage, path: TyPath, log) -> Any:
        # def sh_path_by_pack(
        """ show directory
        """
        _path = str(importlib.resources.files(package).joinpath(path))
        if not _path:
            # print(f"path {path} does not exist in package {package}")
            log.error(f"path {path} does not exist in package {package}")
            return ''
        if os.path.exists(_path):
            # print(f"path {_path} exists")
            log.debug(f"path {_path} exists")
            return _path
        # print(f"path {_path} does not exist")
        log.error(f"path {_path} does not exist")
        return ''

    @classmethod
    def sh_path_by_path_and_prefix(
            cls, package: TyPackage, path: TyPath, log, prefix: TyPath = '') -> Any:
        # def sh_path_by_pack(
        """
        show directory
        """
        return cls.sh_path_by_path(package, os.path.join(prefix, path), log)
